#include <iostream> // pour utiliser std::cout std::endl << >>

#include <GL/glew.h>
#include <GLFW/glfw3.h>

#include <glm/glm.hpp>

#include "main.h"
#include "LisEtCompileLesShaders.h"


#define TAILLE_HORIZONTALE 1024
#define TAILLE_VERTICALE 768

GLFWwindow *MaFenetre;
GLuint MonProgrammeGLSL;
GLuint MonVAO;
GLuint CleDeLargeur, CleDeHauteur;

void Rendu()
{
	glClear( GL_COLOR_BUFFER_BIT );

	glUseProgram(MonProgrammeGLSL);

	glBindVertexArray( MonVAO);
	glDrawArrays( GL_TRIANGLES, 0, 6);
	glBindVertexArray( 0);

	glfwSwapBuffers(MaFenetre);
}

int main( int argc, char *argv[])
{
	MaFenetre = InitialiseUnContexteEtUneFenetreOpenGL();


	MonProgrammeGLSL = LisEtCompileLesShaders( "../shader/TraitementDesSommets.glsl", "../shader/TraitementDesFragments.glsl");
	glUseProgram(MonProgrammeGLSL);

	std::cout << "MonProgrammeGLSL = " << MonProgrammeGLSL << std::endl;

	CleDeLargeur = glGetUniformLocation( MonProgrammeGLSL, "LargeurDeFenetre");
	CleDeHauteur = glGetUniformLocation( MonProgrammeGLSL, "HauteurDeFenetre");

	std::cout << "CleDeLargeur = " << CleDeLargeur << std::endl;
	std::cout << "CleDeHauteur = " << CleDeHauteur << std::endl;

	glUniform1i( CleDeLargeur, TAILLE_HORIZONTALE);
	glUniform1i( CleDeHauteur, TAILLE_VERTICALE);

	GLuint CleDeMaGeometrieSurLaCarteGraphique = TransfertLaGeometrieSurLaCarteGraphique();
	std::cout << "CleDeMaGeometrieSurLaCarteGraphique = " << CleDeMaGeometrieSurLaCarteGraphique << std::endl;

	MonVAO = EncapsuleLaGeometrieDansUnVao( CleDeMaGeometrieSurLaCarteGraphique);
	std::cout<<"MonVAO = "<<MonVAO<< std::endl;

	glClearColor( 0.3, 0.3, 0.3, 0.);

	while (!glfwWindowShouldClose( MaFenetre)) {

		TraiteLesEvenementsClavier( MaFenetre);

		Rendu();

		glfwWaitEvents();
	}

	glDeleteBuffers(1, &CleDeMaGeometrieSurLaCarteGraphique);
	glDeleteVertexArrays(1, &MonVAO);

	glfwTerminate();
}

GLuint TransfertLaGeometrieSurLaCarteGraphique()
{
	glm::vec3 MesSommets[]= { 
		glm::vec3(-0.5, -0.5, -0.0),
		glm::vec3( 0.5, -0.5, 0.0),
		glm::vec3(-0.5,  0.5, 0.0),
		glm::vec3(-0.5,  0.5, 0.0),
		glm::vec3( 0.5, -0.5, 0.0),
		glm::vec3( 0.5,  0.5, 0.0)
	};

	GLuint vboID;
	glGenBuffers( 1, &vboID);
	glBindBuffer(GL_ARRAY_BUFFER, vboID);
	glBufferData(GL_ARRAY_BUFFER, sizeof(MesSommets), MesSommets, GL_STATIC_DRAW);
	glBindBuffer(GL_ARRAY_BUFFER, 0);

	return vboID;
}

void UneFenetreChangeDeTaille( GLFWwindow* UneFenetre, int Largeur, int Hauteur)
{
	std::cout << "Nouvelle taille de fenetre: "<< Largeur << " x " << Hauteur << std::endl;

	glUniform1i( CleDeLargeur, Largeur);
	glUniform1i( CleDeHauteur, Hauteur);

	Rendu();
}

GLuint EncapsuleLaGeometrieDansUnVao(GLuint MaGeometrie)
{
	GLuint vaoID;

	glGenVertexArrays( 1, &vaoID);
	glBindVertexArray(vaoID);

	glEnableVertexAttribArray(0); // ID de l'attribut à configurer

        glBindBuffer(GL_ARRAY_BUFFER, MaGeometrie); // ON ACTIVE LE VBO

       // On lui indique comment lire les donnees
	glVertexAttribPointer(
            0,                  // indice de l'attribut à configurer, correspond à la ligne "layout(location=0) in vec3 MonSommetEnEntree;" dans le vertex shader
            3,                  // nombre de composante par position (x, y, z)
            GL_FLOAT,           // type des composantes
            GL_FALSE,           // normalisation des composantes
            0,                  // decalage des composantes
            (void*)0            // offset des composantes
        );
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindVertexArray(0);

	return vaoID;
}

GLFWwindow* InitialiseUnContexteEtUneFenetreOpenGL()
{
	GLFWwindow* MaFenetre;

	// Initialisation de GLFW
	if( !glfwInit() )
	{
		std::cout << "Echec de l'initialisation de GLFW" << std::endl;
		exit(EXIT_FAILURE);
	}

	glfwWindowHint(GLFW_SAMPLES, 4); // Anti Aliasing
	glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3); // OpenGL 3.3
	glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
	glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
	glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);


	// Ouverture d'une fenêtre en 1024x768
	// et creation d'un contexte OpenGL
	if( !(MaFenetre = glfwCreateWindow(TAILLE_HORIZONTALE, TAILLE_VERTICALE, "Polytech_INFO4", NULL,NULL)) )
	{
		std::cout << "Echec de l'ouverture de fenetre OpenGL" << std::endl;
		glfwTerminate();
		exit(EXIT_FAILURE);
	}
	glfwMakeContextCurrent(MaFenetre);


	// PROCEDURE APPELLEE QUAND LA FENETRE CHANGE DE TAILLE
	glfwSetFramebufferSizeCallback( MaFenetre, UneFenetreChangeDeTaille);

    // Autorise GLFW a recevoir les appuis de touche
	glfwSetInputMode( MaFenetre, GLFW_STICKY_KEYS, GL_TRUE);

    // Initialisation de  GLEW
	glewExperimental = GL_TRUE; 
	if (glewInit() != GLEW_OK) {
		std::cout << "Echec de l'initialisation de GLEW" << std::endl;
		exit(EXIT_FAILURE);
	}


	// Affichage des donnees du contexte OpenGL
	const GLubyte* renderer = glGetString (GL_RENDERER);
	std::cout << "Carte Graphique : " << renderer << std::endl;

	const GLubyte* version = glGetString (GL_VERSION);
	std::cout << "Driver OpenGL : " << version << std::endl;

	const GLubyte* shading_version = glGetString (GL_SHADING_LANGUAGE_VERSION);
	std::cout << "Shading Language: " << shading_version << std::endl;

	return MaFenetre;
}

void TraiteLesEvenementsClavier(GLFWwindow *MaFenetre)
{
	if (glfwGetKey( MaFenetre, GLFW_KEY_A ) == GLFW_PRESS) {
		std::cout << "Touche A\n";
	}
}


